/*
Subject: http://www.byte.com/art/download/rgc.zip
Date: Mon, 05 Feb 1996 13:49:31 -0500 (EST)
From: Justin C Miller <jcmiller@MIT.EDU>

I am the author of the C++ garbage collection article and the
accompanying code in rgc.zip.  I have written a new version
of one of the files in this archive in response to some reader
comments, and was wondering if you could please update the archive
on your web page and FTP site.

Attached is a new version of rgc.h, to replace the file rgc.h in the
rgc.zip archive.

*/

--------
rgc.h
--------
class TRGCObj
{
private:
  int RefCount;

public:

  void RegisterReference() { ++RefCount; }
  int DeregisterReference()
    {
      if (RefCount > 0) --RefCount;
      return RefCount;
    }
  TRGCObj() : RefCount(0) {}
  virtual ~TRGCObj() {}

  TRGCObj(const TRGCObj &notUsed) : RefCount(0) {}
  /* copies will initially start out with 0
     references like any other object */
};

template <class T> class TRGCRef
{
private:
  T *pointer;

  void zeropointer()
    {
      if (pointer) {
        if (pointer->DeregisterReference()==0)
          delete pointer;
        pointer=0;
      }
    }
  void setpointer(T *x)
    {
      if (pointer) zeropointer();
      if (x) {
        x->RegisterReference();
        pointer=x;
      }
    }
public:

  T *getpointer() const { return pointer; }

  TRGCRef() : pointer(0) {}
  TRGCRef(T *p) : pointer(0) { setpointer(p); }
  TRGCRef(const TRGCRef<T> &r) : pointer(0) { setpointer(r.getpointer()); }
  ~TRGCRef() { zeropointer(); }

  TRGCRef<T>& operator= (const TRGCRef<T> &p)
    { setpointer(p.getpointer()); return *this; }
  TRGCRef<T>& operator= (T *x)
    { setpointer(x); return *this; }
  int operator== (const TRGCRef<T> &y) const { return pointer==y.pointer; }
  T& operator* () const { return (*pointer); }
  T* operator-> () const { return pointer; }
  operator T*() const { return pointer; }
};

template <class T> class TRGCRefObj : public TRGCRef<T>, public TRGCObj
{
public:
  TRGCRefObj() : TRGCRef<T>(), TRGCObj() {}
  TRGCRefObj(T *p) : TRGCRef<T>(p), TRGCObj() {}
  TRGCRefObj(const TRGCRef<T> &r) : TRGCRef<T>(r), TRGCObj() {}
  TRGCRefObj(const TRGCRefObj<T> &r) : TRGCRef<T>(r.getpointer()), TRGCObj() {}
  ~TRGCRefObj() {}
};
